package meerkat.voting.output;

import com.google.protobuf.BoolValue;
import meerkat.protobuf.PollingStation.ScannedData;
import meerkat.protobuf.Voting.SignedEncryptedBallot;
import meerkat.rest.Constants;
import meerkat.rest.ProtobufMessageBodyReader;
import meerkat.rest.ProtobufMessageBodyWriter;
import meerkat.voting.output.outputcommands.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.ws.rs.client.Client;
import javax.ws.rs.client.ClientBuilder;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.Response;
import java.io.IOException;

import static meerkat.pollingstation.PollingStationConstants.POLLING_STATION_WEB_SCANNER_SCAN_PATH;

/**
 * Created by hai on 27/06/16.
 */
public class NetworkVirtualPrinter extends AsyncRunnableOutputDevice {

    private Logger logger;
    private SignedEncryptedBallot signedEncryptedBallot;
    private final WebTarget successfulPrintTarget;

    public NetworkVirtualPrinter(String address) {
        super();

        logger = LoggerFactory.getLogger(NetworkVirtualPrinter.class);
        logger.info("A NetworkVirtualPrinter is constructed");

        Client client = ClientBuilder.newClient();
        client.register(ProtobufMessageBodyReader.class);
        client.register(ProtobufMessageBodyWriter.class);
        successfulPrintTarget = client.target(address).path(POLLING_STATION_WEB_SCANNER_SCAN_PATH);
    }


    public void doCommitToBallot(CommitOutputCommand command) {
        logger.debug("entered method doCommitToBallot");
        signedEncryptedBallot = command.getSignedEncryptedBallot();
        command.getCallback().onSuccess(null);
    }


    public void doAudit(AuditOutputCommand command) {
        logger.debug("entered method doAudit");
        signedEncryptedBallot = null;
        command.getCallback().onSuccess(null);
    }


    public void doCastBallot(CastOutputCommand command) {
        logger.debug("entered method doCastBallot");
        ScannedData scannedData = ScannedData.newBuilder()
                .setChannel(null)  //TODO: fill the details for the channel to be able to send here
                .setSignedEncryptedBallot(this.signedEncryptedBallot)
                .build();

        Response response = successfulPrintTarget.request(Constants.MEDIATYPE_PROTOBUF).post(Entity.entity(scannedData, Constants.MEDIATYPE_PROTOBUF));
        BoolValue b = response.readEntity(BoolValue.class);
        response.close();

        if (b.getValue()) {
            command.getCallback().onSuccess(null);
        }
        else {
            command.getCallback().onFailure(new IOException());
        }
    }


    public void doCancel(CancelOutputCommand command) {
        logger.debug("entered method doCancel");
        signedEncryptedBallot = null;
        command.getCallback().onSuccess(null);
    }

}
